# 电涡流传感器模块通信协议

## 电涡流传感器模块通信协议

| **序号** | **修改内容** | **版本** | **日期**     | **修改人** |
|:------:|:--------:|:------:|:----------:|:-------:|
| 1      | 初版       | V1.0   | 2024-12-25 | Hulk    |
|        |          |        |            |         |
|        |          |        |            |         |
|        |          |        |            |         |

### 发包格式

| **包头** | **类型** | **数据长度**    | **数据** | **校验** |
|:------:|:------:|:-----------:|:------:|:------:|
| D5     | 0x03   | Data Length |  Data  | CRC    |

- 数据长度只包含数据部分，不包含包头、类型、数据长度、校验
- CRC求和校验，包含类型、数据长度、数据
- 数据部分为ascii码

### 回包格式

| **包头** |  **状态码**   | **数据长度**    | **数据** | **校验** |
|:------:|:----------:|:-----------:|:------:|:------:|
|   B5   |  0xF0 正常包  | Data Length |  Data  | CRC    |
|   B5   | 0xF1 CRC错误 | Data Length |  Data  | CRC    |
|   B5   | 0xF2 包头错误  | Data Length |  Data  | CRC    |
|   B5   | 0xF3 类型错误  | Data Length |  Data  | CRC    |
|   B5   | 0xF4 包长度错误 | Data Length |  Data  | CRC    |

- 数据长度只包含数据部分，不包含包头、类型、数据长度、校验
- CRC求和校验，包含状态码、数据长度、数据
- 有效数据部分为uint32_t，高字节在前
- 包错误和指令错误时，数据部分为ascii码 `err`

-------------------

## 电涡流传感器模块功能

### 1. 读取电涡流传感器模块数据

- 发送M1指令，读取电涡流传感器模块数据。
  - `D5 03 02 4D 31 83`
- 电涡流传感器模块涡流回复数据
  - `B5 F0 04 01 AE 1B E4 A2`, 有效数据为 `0x01AE1BE4`，转换为`28187620`
  - `B5 F0 04 04 19 C1 FA CC`, 有效数据为 `0x0419C1FAD2`，转换为`17612012242`
- 错误命令（M3指令）回包
  - `B5 F0 03 65 72 72 3C`，有效数据为 `err`
- CRC错误回包
  - `B5 F1 03 65 72 72 3D`, 有效数据为 `err`
- 包头错误回包
  - `B5 F2 03 65 72 72 3E`, 有效数据为 `err`
- 类型错误回包
  - `B5 F3 03 65 72 72 3F`, 有效数据为 `err`
- 数据长度错误回包
  - `B5 F4 03 65 72 72 40`, 有效数据为 `err`

### 2. 读取电涡流传感器模块温度补偿数据

- 发送M2指令，读取电涡流传感器模块数据。
  - `D5 03 02 4D 32 84`
- 电涡流传感器模块温度补偿回复数据
  - `B5 F0 04 00 03 40 85 BC`, 有效数据为 `0x00034085`，转换为`213125`（单位为摄氏度*10）温度为21.3125℃
  - `B5 F0 04 00 03 89 C3 43`, 有效数据为 `0x000389C3`，转换为`231875`（单位为摄氏度*10）温度为23.1875℃
- 错误命令（M3指令）回包
  - `B5 F0 03 65 72 72 3C`，有效数据为 `err`
- CRC错误回包
  - `B5 F1 03 65 72 72 3D`, 有效数据为 `err`
- 包头错误回包
  - `B5 F2 03 65 72 72 3E`, 有效数据为 `err`
- 类型错误回包
  - `B5 F3 03 65 72 72 3F`, 有效数据为 `err`
- 数据长度错误回包
  - `B5 F4 03 65 72 72 40`, 有效数据为 `err`

### 3. 读取数据时间间隔

- 推荐数据时间间隔至少为500ms